import os

from util import pathutils
from util.cfgutils import Cfgini, CfgItem
from dataclasses import dataclass


@dataclass
class PylizDirFolder:
    key: str
    name: str
    path: str



class PylizDir:

    __path: str = None
    __folder_name: str = None
    __folders: [PylizDirFolder] = []
    __ini: Cfgini = None
    __ini_path: str = None
    __ini_initialized = False


    def __init__(self, folder_name: str):
        # Settaggio path
        self.__folder_name = folder_name
        self.__path = pathutils.get_app_home_dir(folder_name)
        # Cartella pyliz
        pathutils.check_path(self.__path, True)
        pathutils.check_path_dir(self.__path)

    def create_ini(self, config_name: str, list_of_items: [CfgItem] = None):
        self.__ini_path = os.path.join(self.__path, config_name)
        self.__ini = Cfgini(self.__ini_path)
        if not self.__ini.exists():
            self.__ini.create(list_of_items)
        self.__ini_initialized = True

    def get_path(self):
        return self.__path

    def add_folder(self, key: str, folder_name: str):
        folder_path = os.path.join(self.__path, folder_name)
        pathutils.create_path(folder_path)
        pathutils.check_path(folder_path, True)
        pathutils.check_path_dir(folder_path)
        self.__folders.append(PylizDirFolder(key, folder_name, folder_path))
        return folder_path

    def add_folder_with_ini(self, key: str, folder_name: str, ini_section: str, ini_key: str):
        folder_path = self.add_folder(key, folder_name)
        self.set_ini_value(ini_section, ini_key, folder_path)


    def get_folder_path(self, key: str):
        for folder in self.__folders:
            if folder.key == key:
                return folder.path
        return None

    def check_for_all_init(self):
        if not self.__ini.exists() or not self.__ini_initialized:
            raise Exception("PylizDirError: Configuration file not initialized or not found")

    def get_ini_value(self, section, key, is_bool=False):
        self.check_for_all_init()
        return self.__ini.read(section, key, is_bool)

    def set_ini_value(self, section, key, value):
        self.check_for_all_init()
        self.__ini.write(section, key, value)


from requests import Response

from network.netrestype import NetResponseType


class NetResponse:

    def __init__(
            self,
            response: Response | None,
            response_type: NetResponseType,
            exception=None
    ):
        self.response = response
        self.hasResponse = self.response is not None
        if self.hasResponse:
            self.code = self.response.status_code
            self.text: str = self.response.text
        else:
            self.code = None
        self.type = response_type
        self.exception = exception
        if self.hasResponse and self.code == 200:
            self.json = self.response.json()

    def __str__(self):
        return ""

    def is_successful(self):
        return self.code == 200

    def is_error(self):
        return self.code != 200

    def get_error(self):
        if self.hasResponse:
            return "(" + str(self.code) + "): " + self.response.text
        else:
            pre = "(" + self.type.value + ") "
            if self.exception is not None:
                pre = pre + str(self.exception)
            return pre



import rich

from ai.controller.mistral_controller import MistralController
from ai.core.ai_inputs import AiInputs
from ai.core.ai_prompts import AiPrompt
from ai.core.ai_setting import AiSettings
from ai.core.ai_source_type import AiSourceType
from media.liz_media import LizMedia
from model.operation import Operation


class AiRunner:

    def __init__(self, settings: AiSettings, inputs: AiInputs):
        self.ai = settings
        self.inputs = inputs

    def __handle_mistral(self) -> Operation[str]:
        controller = MistralController(self.ai.api_key)
        return controller.run(self.ai, self.inputs.prompt, self.inputs.file_path)


    def run(self) -> Operation[str]:
        if self.ai.source_type == AiSourceType.API_MISTRAL:
            return self.__handle_mistral()
        raise NotImplementedError("Source type not implemented yet in AiRunner")


class AiCustomRunner:

    @staticmethod
    def run_for_image(ai_image_setting: AiSettings, ai_text_setting, image_path: str) -> Operation[LizMedia]:
        ai_image_inputs = AiInputs(file_path=image_path, prompt=AiPrompt.VISION_DETAILED.value)
        ai_image_result = AiRunner(ai_image_setting, ai_image_inputs).run()
        rich.print(ai_image_result)
        if not ai_image_result.status:
            return Operation(status=False, error=ai_image_result.error)
        ai_text_inputs = AiInputs(prompt=AiPrompt.EXTRACT_INFO_FROM_VISION_QUERY_JSON.value + ai_image_result.payload)
        ai_text_result = AiRunner(ai_text_setting, ai_text_inputs).run()
        rich.print(ai_text_result)
        if not ai_text_result.status:
            return Operation(status=False, error=ai_text_result.error)
        media = LizMedia(image_path)
        return Operation(status=True, payload=media)


from media.liz_media import LizMedia
from util import fileutils


class LizMediaExporter:

    def __init__(self, media: LizMedia):
        self.media = media

    def export_to_json(self):
        fileutils.write_json_to_file(self.media.path, self.media.file_name + ".json", self.media.to_json_only_ai())



import random
import string
from datetime import datetime


def convert_months_number_to_str(number: int) -> str:
    months = {
        1: "January",
        2: "February",
        3: "March",
        4: "April",
        5: "May",
        6: "June",
        7: "July",
        8: "August",
        9: "September",
        10: "October",
        11: "November",
        12: "December"
    }
    return months.get(number, "Invalid Month")


def gen_random_string(length: int) -> str:
    characters = string.ascii_letters + string.digits
    return ''.join(random.choice(characters) for _ in range(length))


def gen_timestamp_log_name(prefix: str, extension: str):
    return prefix + datetime.now().strftime("%Y%m%d_%H%M%S") + extension





class AiInputs:

    def __init__(self,
                 file_path: str | None = None,
                 prompt: str | None = None,
                ):
        self.file_path = file_path
        self.prompt = prompt



import os
import subprocess
from typing import Callable

from git import Repo

from ai.core.ai_method import AiMethod
from ai.core.ai_models import AiModels
from ai.core.ai_power import AiPower
from ai.core.ai_source import AiSource
from ai.core.ai_source_type import AiSourceType
from model.operation import Operation
from util import pathutils, osutils, fileutils, datautils


# noinspection PyMethodMayBeStatic
class LlamaCpp:

    GITHUB_URL = "https://github.com/ggerganov/llama.cpp.git"

    def __init__(
            self,
            # path_install: str = os.path.join(PylizDir.get_ai_folder(), "llama.cpp"),
            # path_models: str = PylizDir.get_models_folder(),
            # path_logs: str = os.path.join(PylizDir.get_logs_path(), "llama.cpp")
            path_install: str,
            path_models: str,
            path_logs: str
    ):
        # Init paths
        self.path_install = path_install
        self.path_models = path_models
        self.path_logs = path_logs
        self.log_build_folder = os.path.join(self.path_logs, "build")
        pathutils.check_path(self.log_build_folder, True)


    def __clone_repo(self, on_log: Callable[[str], None] = lambda x: None):
        on_log("Cloning LlamaCpp...")
        # check if the folder already exists
        if os.path.exists(self.path_install):
            on_log("LlamaCpp already installed.")
            return
        else:
            on_log("LlamaCpp not installed. Proceeding...")
            pathutils.check_path(self.path_install, True)
        # checking folder
        pathutils.check_path_dir(self.path_install)
        # Cloning github repo
        Repo.clone_from(LlamaCpp.GITHUB_URL, self.path_install)
        on_log("Clone successful.")


    def __check_requirements(self, on_log: Callable[[str], None] = lambda x: None):
        on_log("Checking requirements...")
        make_ok = osutils.is_command_available("make")
        is_os_unix = osutils.is_os_unix()
        if not make_ok:
            raise Exception("Make command not available. Please install make.")
        if not is_os_unix:
            raise Exception("This component (LlamaCPP) is only available on Unix systems.")


    def __build_sources(self, on_log: Callable[[str], None] = lambda x: None):
        on_log("Building sources...")
        risultato = subprocess.run(["make"], check=True, cwd=self.path_install, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        on_log("Build successful.")
        return risultato


    def __crete_build_log(self, risultato, ):
        log_build_name = datautils.gen_timestamp_log_name("llamacpp-", ".txt")
        log_build_path = os.path.join(self.log_build_folder, log_build_name)
        with open(log_build_path, "w") as f:
            f.write(risultato.stdout)
            f.write("***********************************\n")
            f.write(risultato.stderr)

    def __download_llava_local_models(
            self,
            source: AiSource,
            folder: str,
            on_log: Callable[[str], None] = lambda x: None,
            on_progress: Callable[[int], None] = lambda x: None
    ):
        on_log("LLava require " + str(len(source.hg_files)) + " files to download.")
        for hg_file in source.hg_files:
            current_file = os.path.join(folder, hg_file.file_name)
            already_exist = os.path.exists(current_file)
            if already_exist:
                on_log("Model " + hg_file.file_name + " already installed.")
                continue
            on_log("Downloading model " + hg_file.file_name + " from Huggingface...")
            op = fileutils.download_file(hg_file.url, current_file, on_progress)
            if op.status is False:
                raise Exception("Error downloading model: " + op.error)


    def clone_and_build(self, on_log: Callable[[str], None] = lambda x: None):
        self.__clone_repo(on_log)
        self.__check_requirements(on_log)
        build = self.__build_sources(on_log)
        self.__crete_build_log(build)


    def install_llava(
            self,
            power: AiPower,
            on_log: Callable[[str], None] = lambda x: None,
            on_progress: Callable[[int], None] = lambda x: None
    ):
        self.clone_and_build(on_log)
        on_log("Installing LLava...")
        # creating and checking files/folders
        source = AiModels.Llava.get_llava(power, AiSourceType.LOCAL_LLAMACPP)
        folder = os.path.join(self.path_models, source.local_name)
        pathutils.check_path(folder, True)
        pathutils.check_path_dir(folder)
        # Checking available space
        models_size = source.get_files_size_mb()
        has_space = osutils.has_disk_free_space(folder, models_size)
        if not has_space:
            raise Exception("Not enough free space to install LLava.")
        # Downloading files
        self.__download_llava_local_models(source, folder, on_log, on_progress)
        on_log("LLava model installed.")

    def run_llava(
            self,
            power: AiPower,
            image_path: str,
            prompt: str,
    ) -> Operation[str]:
        try:
            # Creating variables and checking requirements
            source = AiModels.Llava.get_llava(power, AiSourceType.LOCAL_LLAMACPP)
            folder = os.path.join(self.path_models, source.local_name)
            if not os.path.exists(folder):
                raise Exception("LLava model not installed.")
            # Run the model
            path_model_file = os.path.join(self.path_models, source.local_name, source.get_ggml_file().file_name)
            path_mmproj_file = os.path.join(self.path_models, source.local_name, source.get_mmproj_file().file_name)
            command = ["./llama-llava-cli", "-m", path_model_file, "--mmproj", path_mmproj_file, "--image", image_path, "-p", prompt ]
            # saving and extracting the result
            log_file = os.path.join(self.path_logs, datautils.gen_timestamp_log_name("llava-result", ".txt"))
            with open(log_file, 'w') as file:
                result = subprocess.run(command, cwd=self.path_install, stdout=file, stderr=subprocess.PIPE)
            if result.returncode != 0:
                raise Exception("Error running LLava: " + result.stderr.decode())
            with open(log_file, 'r') as file:
                return Operation(status=True, payload=file.read())
        except Exception as e:
            return Operation(status=False, error=str(e))







import os
import unittest

from ai.controller.mistral_controller import MistralController
from ai.core.ai_model_list import AiModelList
from ai.core.ai_power import AiPower
from ai.core.ai_setting import AiSettings
from ai.core.ai_source_type import AiSourceType
from ai.llm.remote.service.lmstudioliz import LmStudioLiz
import sys
import os
from dotenv import load_dotenv

from util.cfgutils import CfgItem
from util.pylizdir import PylizDir

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


class TestPylizDir(unittest.TestCase):

    liz = PylizDir(".testPylizdir")

    def setUp(self):
        list = [
            CfgItem("api_keys", "mistral", "cringewhoreads"),
            CfgItem("api_keys", "openai", "cringewhoreads"),
            CfgItem("general", "test", "valueFromGeneral2"),
        ]
        self.liz.create_ini("testFromPython.ini", list)
        print("SDone setup")

    def print_values(self):
        print(self.liz.get_ini_value("api_keys", "mistral"))
        print(self.liz.get_ini_value("api_keys", "openai"))
        print(self.liz.get_ini_value("general", "test"))

    def testGet(self):
        self.print_values()

    def testSet(self):
        self.print_values()
        self.liz.set_ini_value("api_keys", "mistral", "newMistralKey")
        self.print_values()




if __name__ == "__main__":
    unittest.main()


import base64
import requests
import os
from mistralai import Mistral


class Mistraliz:


    def __init__(self, api_key: str):
        self.api_key = api_key


    def run_pixstral_vision(self, model_name: str, image_path: str, prompt: str) -> str:
        with open(image_path, "rb") as image_file:
            base_image = base64.b64encode(image_file.read()).decode('utf-8')
        client = Mistral(api_key=self.api_key)
        messages = [
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": prompt
                    },
                    {
                        "type": "image_url",
                        "image_url": f"data:image/jpeg;base64,{base_image}"
                    }
                ]
            }
        ]
        chat_response = client.chat.complete(
            model=model_name,
            messages=messages
        )
        return chat_response.choices[0].message.content

    def run_open_mistral(self, model_name: str, prompt: str) -> str:
        client = Mistral(api_key=self.api_key)
        chat_response = client.chat.complete(
            model=model_name,
            messages=[
                {
                    "role": "user",
                    "content": prompt,
                },
            ]
        )
        return chat_response.choices[0].message.content



from util import osutils, pathutils
from util.loggiz import Loggiz


class Easinit:
    def __init__(
            self,
            app_name: str,
            console_log=False,
            file_log=False
    ):
        self.console_log = console_log
        self.file_log = file_log
        self.app_name = app_name
        self.home_folder_name = "." + self.app_name.lower().replace(" ", "_")
        self.setting_file_name = "config.ini"
        self.dir_app = pathutils.get_app_home_dir(self.home_folder_name)

    def create(self):
        pass

    def set_logging(self):
        Loggiz.setup(
            app_name=self.app_name,
            setup_console=False,
            file_log_base_path=self.dir_app,
            file_log_folder_name="logs",
        )

    def set_config(self):
        pass


from enum import Enum


class AiMethod(Enum):
    # LLAVA_OLLAMA = "LLAVA with ollama server"
    # LLAVA_LOCAL_LLAMACPP = "LLAVA with local power"
    # LLAVA_LLAMACPP_AFTER_OLLAMA = "LLAVA with ollama and llamacpp"

    LLAVA_INNER_JSON = "LLAVA with inner json"
    LLAVA_NORMALE_JSON = "LLAVA with normale json"



from ai.core.ai_method import AiMethod
from ai.core.ai_model_list import AiModelList
from ai.core.ai_models import AiModels
from ai.core.ai_power import AiPower
from ai.core.ai_prompts import AiPrompt
from ai.core.ai_scan_settings import AiScanSettings
from ai.core.ai_source import AiSource
from ai.core.ai_source_type import AiSourceType


class AiSettings:
    def __init__(
            self,
            model: AiModelList,
            source_type: AiSourceType,
            power: AiPower,
            remote_url: str | None = None,
            api_key: str | None = None,
    ):
        self.source: AiSource | None = None
        self.api_key = api_key
        self.model_name = model
        self.source_type = source_type
        self.remote_url = remote_url
        self.power = power

        self.check()
        self.setup()

    def setup(self):
        if self.model_name == AiModelList.LLAVA:
            self.source = AiModels.Llava.get_llava(self.power, self.source_type)
        if self.model_name == AiModelList.OPEN_MISTRAL:
            self.source = AiModels.Mistral.get_open_mistral()
        if self.model_name == AiModelList.PIXSTRAL:
            self.source = AiModels.Mistral.get_pixstral()


    def check(self):
        if self.source_type == AiSourceType.OLLAMA_SERVER and self.remote_url is None:
            raise ValueError("Remote URL is required for Ollama Server.")
        if self.source_type == AiSourceType.LMSTUDIO_SERVER and self.remote_url is None:
            raise ValueError("Remote URL is required for LM Studio Server.")
        if self.source_type == AiSourceType.API_MISTRAL and self.api_key is None:
            raise ValueError("API Key is required for Mistral API.")




import os
import unittest

from ai.llm.remote.service.lmstudioliz import LmStudioLiz
import sys
import os
from dotenv import load_dotenv
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


class TestLmStudio(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        print("Setting up test...")


    def test1(self):
        liz = LmStudioLiz(os.getenv('LMSTUDIO_URL'))
        obj = liz.get_loaded_models()
        print(obj.data[0].id)

    def test2(self):
        import http.client
        conn = http.client.HTTPConnection("192.168.0.253", 1234)
        conn.request("GET", "/v1/models")
        response = conn.getresponse()

        print(response.status, response.reason)
        data = response.read()
        print(data)



if __name__ == "__main__":
    unittest.main()


from typing import List

from ai.core.hg_file import HgFile
from model.file_type import FileType
from network import netutils
from util import unitutils


class AiSource:

    def __init__(
            self,
            ollama_name: str | None = None,
            lmstudio_name: str | None = None,
            mistral_name: str | None = None,
            local_name: str | None = None,
            url: str | None = None,
            hg_files: List[HgFile] | None = None,
    ):
        self.url = url
        self.hg_files = hg_files
        self.ollama_name = ollama_name
        self.local_name = local_name
        self.lmstudio_name = lmstudio_name
        self.mistral_name = mistral_name

    def get_ggml_file(self) -> HgFile:
        for hg_file in self.hg_files:
            if hg_file.file_type == FileType.HG_GGML:
                return hg_file
        raise Exception("No ggml file found in the source.")

    def get_mmproj_file(self) -> HgFile:
        for hg_file in self.hg_files:
            if hg_file.file_type == FileType.HG_MMPROJ:
                return hg_file
        raise Exception("No mmproj file found in the source.")

    def get_files_size_mb(self) -> float:
        total = 0.0
        for hg_file in self.hg_files:
            size_byte = hg_file.get_file_size_byte()
            size_mb = unitutils.convert_byte_to_mb(size_byte)
            total += size_mb
        return total




import os
import google.generativeai as genai


class GeminiController:

    def __init__(self, key: str):
        genai.configure(api_key=key)

    def upload(self, path: str, file_name: str):
        sample_file = genai.upload_file(path=path, display_name=file_name)








import os
import unittest

from ai.core.ai_model_list import AiModelList
from ai.core.ai_power import AiPower
from ai.core.ai_setting import AiSettings
from ai.core.ai_source_type import AiSourceType
from ai.llm.remote.service.lmstudioliz import LmStudioLiz
import sys
import os
from dotenv import load_dotenv

from ai.util.ai_runner import AiCustomRunner

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


class TestAiImage(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        print("Setting up test...")


    def test1(self):
        image = os.getenv('LOCAL_IMAGE_FOR_TEST')
        api_key = os.getenv('MISTRAL_API_KEY')
        ai_image_setting = AiSettings(
            model=AiModelList.PIXSTRAL,
            source_type=AiSourceType.API_MISTRAL,
            power=AiPower.MEDIUM,
            api_key=api_key,
        )
        ai_text_setting = AiSettings(
            model=AiModelList.OPEN_MISTRAL,
            source_type=AiSourceType.API_MISTRAL,
            power=AiPower.LOW,
            api_key=api_key,
        )
        media = AiCustomRunner.run_for_image(ai_image_setting, ai_text_setting, image)



if __name__ == "__main__":
    unittest.main()


import os
import shutil
import subprocess
import platform


def get_folder_size_mb(path) -> float:
    """
    Get the size of a folder in megabytes
    :param path: path to the folder
    :return: size of the folder in megabytes
    """
    # Inizializza la dimensione totale a 0
    total_size = 0
    # Scansione delle cartelle e dei file all'interno del percorso dato
    for root, dirs, files in os.walk(path):
        # Aggiungi le dimensioni dei file alla dimensione totale
        for file in files:
            file_path = os.path.join(root, file)
            total_size += os.path.getsize(file_path)
    # Converti la dimensione totale in megabyte (MB)
    total_size_mb = total_size / (1024 * 1024)
    return total_size_mb


def open_system_folder(path):
    """
    Open a system folder in the default file explorer
    :param path: path to the folder
    :return:
    """
    if not os.path.exists(path):
        raise FileNotFoundError(f"Path {path} does not exist!")
    if os.name == 'nt':  # For Windows
        subprocess.Popen(['explorer', path])
    elif os.name == 'posix':  # For Linux, Mac
        subprocess.Popen(['open', path])
    else:
        raise OSError("Unsupported OS")


def has_disk_free_space(path_of_disk, mb_free):
    """
    Check if a disk has enough free space in megabytes
    :param path_of_disk: The path of the disk to check
    :param mb_free: The minimum amount of free space in megabytes
    :return: True if the disk has enough free space, False otherwise
    """
    stat = shutil.disk_usage(path_of_disk)
    spazio_disponibile_mb = stat.free / (1024 * 1024)
    if spazio_disponibile_mb > mb_free:
        return True
    else:
        return False


def get_free_space_mb(directory) -> float:
    """
    Get the free space in megabytes of a directory
    :param directory: path to the directory
    :return: free space in megabytes
    """
    statvfs = os.statvfs(directory)
    # Calculate the free space in bytes and convert to megabytes
    free_space = statvfs.f_frsize * statvfs.f_bavail / (1024 * 1024)
    return free_space


def get_directory_size(path) -> float:
    """
    Get the size of a directory in megabytes
    :param path: path to the directory
    :return: size of the directory in megabytes
    """
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            total_size += os.path.getsize(fp)
    return total_size / (1024 * 1024)


def check_move_dirs_free_space(src_path, dst_path) -> bool:
    """
    Check if there is enough free space in the destination directory to move the source directory
    :param src_path: The path of the source directory
    :param dst_path: The path of the destination directory
    :return: True if there is enough free space, False otherwise
    """
    # Calculate the size of the source directory
    src_size_mb = get_directory_size(src_path)
    # Get the free space of the destination directory
    free_space_mb = get_free_space_mb(dst_path)
    # Check if there is enough space
    return free_space_mb >= src_size_mb


def is_command_available_with_run(command: str) -> bool:
    """
    Check if a command is available in the system
    :param command: The command to check
    :return: True if the command is available, False otherwise
    """
    try:
        subprocess.run([command], check=True)
        return True
    except FileNotFoundError:
        return False


def is_command_available(command: str) -> bool:
    return shutil.which(command) is not None


def is_os_unix() -> bool:
    """
    Check if the operating system is Unix-based (Linux, MacOS)
    :return: True if the operating system is Unix-based, False otherwise
    """
    current_os = platform.system()
    return current_os in ["Linux", "Darwin"]




import os
import unittest

from ai.llm.local.llamacpplib import LlamaCppLib
from old_code.pylizdir_OLD import PylizDir




class TestLlamaCPPLib(unittest.TestCase):

    def setUp(self):
        print("Setting up test...")
        PylizDir.create()
        path_install: str = os.path.join(PylizDir.get_ai_folder(), "llama.cpp")
        path_models: str = PylizDir.get_models_folder()
        path_logs: str = os.path.join(PylizDir.get_logs_path(), "llama.cpp")


    def test1(self):
        LlamaCppLib.run_llama3("What is SpaceX ?")


if __name__ == "__main__":
    unittest.main()


import base64
import os

from ai.core.ai_setting import AiSettings
from ai.core.ai_source_type import AiSourceType
from ai.llm.local.llamacpp import LlamaCpp
from ai.llm.remote.service.lmstudioliz import LmStudioLiz
from ai.llm.remote.service.ollamaliz import Ollamaliz
from model.operation import Operation
from old_code.pylizdir_OLD import PylizDir


class LlavaController:

    def __init__(self, settings: AiSettings):
        self.settings = settings

    def __run_from_ollama(self, image_path: str, prompt: str) -> Operation[str]:
        ollama = Ollamaliz(self.settings.remote_url)
        model_name = self.settings.source.ollama_name
        with open(image_path, "rb") as image_file:
            image_base_64 = base64.b64encode(image_file.read()).decode('utf-8')
        llava_result = ollama.llava_query(prompt, image_base_64, model_name)
        if not llava_result.is_op_ok():
            return Operation(status=False, error=llava_result.error)
        return Operation(status=True, payload=llava_result.payload.response)

    def __run_from_lm_studio(self, image_path: str, prompt: str) -> Operation[str]:
        lm_studio = LmStudioLiz(self.settings.remote_url)
        model_name = self.settings.source.lmstudio_name
        try:
            if not lm_studio.has_model_loaded(model_name):
                return Operation(status=False, error=f"Model {model_name} is not loaded in LM Studio.")
            with open(image_path, "rb") as image_file:
                image_base_64 = base64.b64encode(image_file.read()).decode('utf-8')
            response = lm_studio.send_vision_prompt(image_base_64, prompt)
            return Operation(status=True, payload=response)
        except Exception as e:
            return Operation(status=False, error=str(e))


    def __run_from_local_llamacpp(self, image_path: str, prompt: str) -> Operation[str]:
        PylizDir.create()
        path_install: str = os.path.join(PylizDir.get_ai_folder(), "llama.cpp")
        path_models: str = PylizDir.get_models_folder()
        path_logs: str = os.path.join(PylizDir.get_logs_path(), "llama.cpp")
        obj = LlamaCpp(path_install, path_models, path_logs)
        obj.install_llava(self.settings.power, lambda x: None, lambda x: None)
        llava_result = obj.run_llava(self.settings.power, image_path, prompt)
        if not llava_result.is_op_ok():
            return Operation(status=False, error=llava_result.error)
        return Operation(status=True, payload=llava_result.payload)


    # def get_liz_media_from_json(self, output: str) -> Operation[LizMedia]:
    #     raise NotImplementedError("This method is not implemented.")


    def run_and_get_vanilla_json(self, image_path: str, prompt: str) -> Operation[str]:
        if self.settings.source_type == AiSourceType.LMSTUDIO_SERVER:
            return self.__run_from_lm_studio(image_path, prompt)
        elif self.settings.source_type == AiSourceType.OLLAMA_SERVER:
            return self.__run_from_ollama(image_path, prompt)
        elif self.settings.source_type == AiSourceType.LOCAL_LLAMACPP:
            return self.__run_from_local_llamacpp(image_path, prompt)
        else:
            raise NotImplementedError("This source type for llava is not implemented.")


import os
from typing import Callable, List

from util import fileutils


def get_home_dir():
    """
    Get the home directory of the current user
    :return: The home directory of the current user
    """
    return os.path.expanduser("~")


def get_app_home_dir(app_name, create_if_not: bool = True):
    """
    create and return the home directory for the application
    :param create_if_not: boolean flag to create the directory if it does not exist
    :param app_name:  name of the application
    :return: home directory for the application
    """
    home_dir = get_home_dir()
    app_home_dir = os.path.join(home_dir, app_name)
    if create_if_not:
        if not os.path.exists(app_home_dir):
            os.makedirs(app_home_dir)
    return app_home_dir


def create_path(path):
    """
    Create a path if it does not exist
    :param path: path to create
    :return:
    """
    if not os.path.exists(path):
        os.makedirs(path)


def check_path(path, create_if_not: bool = False) -> bool:
    """
    Check if the path exists and is readable
    :param path: path to check
    :param create_if_not: create the path if it does not exist
    :return: True if the path exists and is readable, False otherwise
    """
    if not os.path.exists(path):
        if create_if_not:
            os.makedirs(path)
            return False
        else:
            return False
    if not os.access(path, os.R_OK):
        return False
    return True


def check_path_dir(path):
    """
    Check if the path:
    - exists and is readable and writable
    - is a directory
    If any of the conditions is not met, an exception is raised
    :param path:  path to check
    :return:
    """
    # Check if the path exists and is readable
    if not os.path.exists(path):
        raise IOError(f'Path {path} does not exist!')
    if not os.access(path, os.R_OK):
        raise PermissionError(f'Path {path} is not readable!')
    if not os.access(path, os.W_OK):
        raise PermissionError(f'Path {path} is not writable!')
    # Check if the path is a directory
    if not os.path.isdir(path):
        raise NotADirectoryError(f'Path {path} is not a directory!')
    if not os.access(path, os.W_OK):
        raise PermissionError(f'Path {path} is not writable!')


def check_path_file(path: str):
    """
    Check if the path:
    - exists and is readable and writable
    - is a file
    If any of the conditions is not met, an exception is raised
    :param path:  path to check
    :return:
    """
    if not os.path.exists(path):
        raise FileNotFoundError(f"File {path} does not exist!")
    if not os.access(path, os.R_OK):
        raise PermissionError(f"File {path} is not readable!")
    if not os.access(path, os.W_OK):
        raise PermissionError(f"File {path} is not writable!")
    if not os.path.isfile(path):
        raise NotADirectoryError(f"Path {path} is not a file!")


def get_second_to_last_directory(path):
    """
    Get the second to last directory in a path as a string
    :param path: path string to get the second to last directory from
    :return: the second to last directory in the path as a string (only name not path).
    """
    # Divide il percorso in una lista di componenti
    path_components = os.path.normpath(path).split(os.sep)
    # Controlla che il percorso abbia almeno due componenti
    if len(path_components) < 2:
        return None
    # Restituisce il secondo componente dal fondo della lista
    return path_components[-2]


def count_pathsub_files(path):
    """
    Count the number of files in a path including subdirectories
    :param path: path to count files from
    :return: the number of files in the path including subdirectories
    """
    count = 0
    for root, dirs, files in os.walk(path):
        count += len(files)
    return count


def count_pathsub_dirs(path):
    """
    Count the number of directories in a path including subdirectories
    :param path: path to count directories from
    :return: the number of directories in the path including subdirectories
    """
    count = 0
    for root, dirs, files in os.walk(path):
        count += len(dirs)
    return count


def count_pathsub_elements(path):
    """
    Count the number of files and directories in a path including subdirectories
    :param path: path to count files and directories from
    :return: the number of files and directories in the path including subdirectories
    """
    count = 0
    for root, dirs, files in os.walk(path):
        count += len(files) + len(dirs)
    return count


def scan_directory(path: str, on_file, on_folder):
    """
    Scan a directory and call the on_file and on_folder functions for each file and folder found
    :param path: The path to scan
    :param on_file: function to call for each file found
    :param on_folder: function to call for each folder found
    :return:
    """
    for root, dirs, files in os.walk(path):
        for file in files:
            on_file(file)
        for dir in dirs:
            scan_directory(os.path.join(root, dir), on_file, on_folder)


def scan_directory_match_bool(path: str, to_be_add: Callable[[str], bool]) -> List[str]:
    """
    Scan a directory and return a list of files that match the to_be_add function
    :param path: The path to scan
    :param to_be_add: function to call for each file found to check if it should be added to the final list.
    :return: list of files that match the to_be_add function
    """
    matching_files = []
    for root, dirs, files in os.walk(path):
        for file in files:
            file_path = os.path.join(root, file)
            if to_be_add(file_path):
                matching_files.append(file_path)
    return matching_files


def dir_contains_image(path: str):
    """
    Check if a directory contains an image file
    :param path: path to the directory to check
    :return: True if the directory contains an image file, False otherwise
    """
    files = scan_directory_match_bool(path, fileutils.is_image_file)
    return len(files) > 0


def dir_contains_video(path: str):
    """
    Check if a directory contains a video file
    :param path: path to the directory to check
    :return: True if the directory contains a video file, False otherwise
    """
    files = scan_directory_match_bool(path, fileutils.is_video_file)
    return len(files) > 0





from network.netutils import exec_get, exec_post

EAGLE_PORT = "41595"
EAGLE_LOCALHOST_URL = "http://localhost:" + EAGLE_PORT


class EagleApi:

    def __init__(self, url: str = EAGLE_LOCALHOST_URL):
        self.url = url
        pass

    def get_app_info(self):
        api_url = self.url + "/api/application/info"
        return exec_get(api_url)

    def add_image_from_path(
            self,
            path: str,
            name: str,
            tags: list[str],
            annotation: str,
            modification_time: float,
    ):
        api_url = self.url + "/api/item/addFromPath"
        payload = {
            "path": path,
            "name": name,
            "tags": tags,
            "annotation": annotation,
            "modification_time": modification_time
        }
        return exec_post(api_url, payload, False)



import os
import unittest

import rich

from ai.llm.local.llamacpp import LlamaCpp
from ai.core.ai_power import AiPower
from ai.core.ai_prompts import prompt_llava_json
from old_code.pylizdir_OLD import PylizDir


def log(message: str):
    rich.print(message)


def progress(percent: int):
    rich.print(f"Progress: {percent}%")


class TestLlamaCPP(unittest.TestCase):

    def setUp(self):
        print("Setting up test...")
        PylizDir.create()
        path_install: str = os.path.join(PylizDir.get_ai_folder(), "llama.cpp")
        path_models: str = PylizDir.get_models_folder()
        path_logs: str = os.path.join(PylizDir.get_logs_path(), "llama.cpp")
        self.obj = LlamaCpp(path_install, path_models, path_logs)

    def test_install_llava(self):
        try:
            self.obj.install_llava(AiPower.LOW, log, progress)
        except Exception as e:
            self.fail(e)

    def test_run_llava(self):
        try:
            result = self.obj.run_llava(AiPower.LOW, "/Users/gabliz/Pictures/obama343434333.jpg", prompt_llava_json)
            print(result)
        except Exception as e:
            self.fail(e)


if __name__ == "__main__":
    unittest.main()


from enum import Enum

from ai.core.ai_model_list import AiModelList


class AiSourceType(Enum):
    OLLAMA_SERVER = "Ollama server",
    LMSTUDIO_SERVER = "LMM studio server",
    LOCAL_LLAMACPP = "Local (Llamacpp)",
    LOCAL_LLAMACPP_LIB = "Local (Official Llamacpp Library)"
    API_MISTRAL = "Mistral API"
    API_GEMINI = "Gemini API"


    def get_vision_models(self) -> list[AiModelList]:
        if self == AiSourceType.API_MISTRAL:
            return [
                AiModelList.PIXSTRAL
            ]

    def get_text_models(self) -> list[AiModelList]:
        if self == AiSourceType.API_MISTRAL:
            return [
                AiModelList.OPEN_MISTRAL
            ]

    def get_model_list(self) -> list[AiModelList]:
        array = []
        array.extend(self.get_vision_models())
        array.extend(self.get_text_models())
        return array




import configparser
import os
from typing import List
from collections import OrderedDict


class CfgItem:
    def __init__(self, section, key, value):
        self.section = section
        self.key = key
        self.value = value


class Cfgini:
    def __init__(self, path_to_ini):
        self.config = None
        self.path = path_to_ini

    def exists(self):
        return os.path.exists(self.path)

    def create(self, items: List[CfgItem] | None = None):
        self.config = configparser.ConfigParser()
        list_of_sections = set()

        for item in items:
            list_of_sections.add(item.section)

        for section in list_of_sections:
            self.config.add_section(section)

        for item in items:
            self.config.set(item.section, item.key, item.value)

        try:
            with open(self.path, 'w') as configfile:
                self.config.write(configfile)
            # print("Configuration file created in: ", self.path)
        except Exception as e:
            print("Error while creating configuration file: ", e)

    def read(self, section, key, is_bool=False):
        self.config = configparser.ConfigParser()
        self.config.read(self.path)
        if is_bool:
            return self.config.getboolean(section, key)
        return self.config.get(section, key)

    def write(self, section, key, value):
        self.config = configparser.ConfigParser()
        self.config.read(self.path)
        self.config.set(section, key, value)
        with open(self.path, 'w') as configfile:
            self.config.write(configfile)






import json
from typing import Callable

import ollama
from ollama import Client

from network.netres import NetResponse
from network.netutils import exec_get, exec_post

OLLAMA_PORT = "11434"
OLLAMA_HTTP_LOCALHOST_URL = "http://localhost:" + OLLAMA_PORT

# https://github.com/ollama/ollama-python
# https://github.com/ollama/ollama/blob/main/docs/api.md


class OllamaApiLegacy:
    def __init__(self):
        pass

    @staticmethod
    def check_ollama_status(url) -> NetResponse:
        return exec_get(url)

    @staticmethod
    def get_installed_models(url) -> NetResponse:
        api_url = url + "/api/tags"
        return exec_get(api_url)

    @staticmethod
    def send_query(
            url: str,
            prompt: str,
            model_name: str,
    ) -> NetResponse:
        api_url = url + "/api/generate"
        payload = {
            "model": model_name,
            "prompt": prompt,
            "stream": False
        }
        return exec_post(api_url, payload, False)

    @staticmethod
    def send_llava_query(
            url: str,
            prompt: str,
            image_base_64: str,
            model_name: str,
    ) -> NetResponse:
        api_url = url + "/api/generate"
        payload = {
            "model": model_name,
            "prompt": prompt,
            "images": [image_base_64],
            "format": "json",
            "stream": False
        }
        return exec_post(api_url, payload, False)


class Ollamapi:

    def __init__(self, url: str):
        self.url = url
        self.client = ollama.client = Client(host=url)

    def get_models_list(self):
        mappings = self.client.list()
        json_str = json.dumps(mappings)
        return json_str

    def download_model(self, name: str, en_stream: bool, callback: Callable[[str], None] | None = None):
        stream = self.client.pull(
            model='name',
            stream=en_stream,
        )
        if en_stream:
            for data in stream:
                if callback:
                    callback(data["status"])




class AiScanSettings:

    def __init__(
            self,
            ai_tags: bool = False,
            ai_file_metadata: bool = False,
            ai_comment: bool = False,
            ai_rename: bool = False,
            ai_ocr: bool = False,
    ):
        self.ai_tags = ai_tags
        self.ai_file_metadata = ai_file_metadata
        self.ai_comment = ai_comment
        self.ai_rename = ai_rename
        self.ai_ocr = ai_ocr

    def set_all_true(self):
        self.ai_tags = True
        self.ai_file_metadata = True
        self.ai_comment = True
        self.ai_rename = True
        self.ai_ocr = True



import json
import os
from typing import Optional, List

from util import fileutils


class LizMedia:

    def __init__(self, path: str):

        # file info
        self.path = path
        self.file_name = os.path.basename(self.path)
        self.extension = os.path.splitext(path)[1].lower()
        self.creation_time = fileutils.get_file_c_date(self.path)
        self.creation_time_timestamp: float = self.creation_time.timestamp()
        self.year, self.month, self.day = self.creation_time.year, self.creation_time.month, self.creation_time.day
        self.size_byte = os.path.getsize(self.path)
        self.size_mb = self.size_byte / (1024 * 1024)

        # type of media
        self.is_image = fileutils.is_image_file(self.path)
        self.is_video = fileutils.is_video_file(self.path)

        # ai info
        self.ai_ocr_text: Optional[List[str]] = None
        self.ai_file_name: Optional[str] = None
        self.ai_description: Optional[str] = None
        self.ai_tags: Optional[List[str]] = None
        self.ai_scanned: bool = False

        # video info
        if self.is_video:
            self.duration: Optional[float] = self.get_video_duration()
            self.frame_rate: Optional[float] = self.get_video_frame_rate()
        else:
            self.duration = None
            self.frame_rate = None

    def get_desc_plus_text(self):
        if self.ai_ocr_text is not None and len(self.ai_ocr_text) > 0:
            return self.ai_description + " This media includes texts: " + self.ai
        return self.ai_description

    def get_video_duration(self) -> float:
        # Logica per ottenere la durata del video (da implementare)
        return 0.0

    def get_video_frame_rate(self) -> float:
        # Logica per ottenere il frame rate del video (da implementare)
        return 0.0

    def to_dict_only_ai(self):
        return {
            "path": self.path,
            "file_name": self.file_name,
            "extension": self.extension,
            "creation_time_timestamp": self.creation_time_timestamp,
            "size_byte": self.size_byte,
            "size_mb": self.size_mb,
            "ai_ocr_text": self.ai_ocr_text,
            "ai_file_name": self.ai_file_name,
            "ai_description": self.ai_description,
            "ai_tags": self.ai_tags,
            "ai_scanned": self.ai_scanned,
        }

    def to_json_only_ai(self):
        return json.dumps(self.to_dict_only_ai(), indent=4)


from typing import Any
from dataclasses import dataclass
import json


@dataclass
class Data:
    version: str
    prereleaseVersion: str
    buildVersion: str
    execPath: str
    platform: str

    @staticmethod
    def from_dict(obj: Any) -> 'Data':
        _version = str(obj.get("version", None))
        _prereleaseVersion = str(obj.get("prereleaseVersion", None))
        _buildVersion = str(obj.get("buildVersion", None))
        _execPath = str(obj.get("execPath", None))
        _platform = str(obj.get("platform", None))
        return Data(_version, _prereleaseVersion, _buildVersion, _execPath, _platform)


@dataclass
class EagleDto:
    status: str
    data: Data | None
    data2: str | None

    @staticmethod
    def from_dict(obj: Any) -> 'EagleDto':
        _status = str(obj.get("status"))
        _data = Data.from_dict(obj.get("data") if "data" in obj else None)
        return EagleDto(_status, _data, None)

    @staticmethod
    def from_dict2(obj: Any) -> 'EagleDto':
        _status = str(obj.get("status"))
        _data2 = obj.get("data") if "data" in obj else None
        return EagleDto(_status, None, _data2)



import moviepy.editor as mp
import whisper

from network.ssl import ignore_context_ssl


def transcribe(
    video_path: str,
    audio_path: str,
    model_path: str,
    transcription_path: str | None = None
):

    ignore_context_ssl()

    # Estrarre l'audio dal video
    video = mp.VideoFileClip(video_path)
    video.audio.write_audiofile(audio_path)

    # Caricare il modello Whisper
    model = whisper.load_model(model_path)

    # Trascrivere l'audio
    result = model.transcribe(audio_path)
    if transcription_path is not None:
        # Salvare la trascrizione su un file di test
        with open(transcription_path, 'w') as f:
            f.write(result["text"])
    return result



from enum import Enum

from model.file_type import FileType
from network import netutils


class HgFile:
    def __init__(
            self,
            file_name: str,
            url: str,
            file_type: FileType
    ):
        self.file_name = file_name
        self.url = url
        self.file_type = file_type

    def get_file_size_byte(self) -> int:
        return netutils.get_file_size_byte(self.url)




from enum import Enum

prompt_llava_json = """
Analyze the image thoroughly and provide a detailed description of every visible element. Return a json including the following information:
- "description": a detailed description of the image (minimum 15-20 words), considering colors, objects, actions, and any other relevant details.
- "tags": a list of tags that describe the image. Include specific objects, actions, locations, and any discernible themes. (minimum 5 maximum 10 tags)
- "text": a list of all the text found in the image (if any).
- "filename": phrase that summarizes the image content (maximum 30 characters).
"""

extract_info_from_image = """
Analyze the following text that contains a description of image. Return a json including the following information:
- "tags": a list of tags that describe the image. Include specific objects, actions, locations, and any discernible themes. (minimum 5 maximum 10 tags)
- "text": a list of all the text found in the image (if specified).
- "filename": phrase that summarizes the image content (maximum 30 characters).
"""

prompt_llava_detailed_STEP1 = """
Analyze the image thoroughly and provide a detailed description of every visible element. 
If there are people, try to recognize them. If there are objects, try to identify them.
If the are texts, try to read them.
"""


class AiPrompt(Enum):
    LLAVA_INNER_JSON = prompt_llava_json
    VISION_DETAILED = prompt_llava_detailed_STEP1
    EXTRACT_INFO_FROM_VISION_QUERY_JSON = extract_info_from_image






import logging

import requests

from network.netres import NetResponse
from network.netrestype import NetResponseType


logging.basicConfig(level=logging.DEBUG)


def test_with_head(url: str) -> bool:
    try:
        response = requests.head(url, timeout=5)
        return response.status_code < 400
    except requests.RequestException as e:
        return False


def exec_get(url: str, sec_timeout: int | None = 10) -> NetResponse:
    try:
        response = requests.get(url, allow_redirects=True)
        if response.status_code == 200:
            return NetResponse(response, NetResponseType.OK200)
        else:
            return NetResponse(response, NetResponseType.ERROR)
    except requests.ConnectionError as e:
        return NetResponse(None, NetResponseType.CONNECTION_ERROR, e)
    except requests.Timeout as e:
        return NetResponse(None, NetResponseType.TIMEOUT, e)
    except requests.RequestException as e:
        return NetResponse(None, NetResponseType.REQUEST_ERROR, e)


def exec_post(
        url: str,
        payload,
        verify_bool: bool,
) -> NetResponse:
    try:
        response = requests.post(url, json=payload, verify=verify_bool, allow_redirects=True)
        if response.status_code == 200:
            return NetResponse(response, NetResponseType.OK200)
        else:
            return NetResponse(response, NetResponseType.ERROR)
    except requests.ConnectionError as e:
        return NetResponse(None, NetResponseType.CONNECTION_ERROR, e)
    except requests.Timeout as e:
        return NetResponse(None, NetResponseType.TIMEOUT, e)
    except requests.RequestException as e:
        return NetResponse(None, NetResponseType.REQUEST_ERROR, e)


def get_file_size_byte(url: str, exception_on_fail: bool = False) -> int:
    try:
        response = requests.head(url, timeout=5, allow_redirects=True)
        response.raise_for_status()
        file_size = response.headers.get('content-length', 0)
        if file_size is None:
            if exception_on_fail:
                raise ValueError("Unable to get file size for url: " + url)
            return -1
        return int(file_size)
    except requests.RequestException as e:
        if exception_on_fail:
            raise ValueError("Unable to get file size for url: " + url + ": " + str(e))
        return -1


from huggingface_hub import hf_hub_download
from llama_cpp import Llama

from old_code.pylizdir_OLD import PylizDir

# https://github.com/abetlen/llama-cpp-python


class LlamaCppLib:

    @staticmethod
    def run_llama3(prompt: str):
        model_name = "Orenguteng/Llama-3-8B-Lexi-Uncensored-GGUF"
        model_file = "Lexi-Llama-3-8B-Uncensored_Q4_K_M.gguf"
        model_path = hf_hub_download(model_name,
                                     filename=model_file,
                                     local_dir=PylizDir.get_models_folder(),
                                     )
        llm = Llama(model_path, n_gpu_layers=1)
        response = llm(prompt)
        print(response)




import json
from typing import List, Any, Dict

class LmStudioModel:
    def __init__(self, id: str, object_type: str, owned_by: str):
        self.id = id
        self.object_type = object_type
        self.owned_by = owned_by

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'LmStudioModel':
        return cls(
            id=data['id'],
            object_type=data['object'],
            owned_by=data['owned_by']
        )

    def to_dict(self) -> Dict[str, Any]:
        return {
            'id': self.id,
            'object': self.object_type,
            'owned_by': self.owned_by
        }

class LmStudioModelList:
    def __init__(self, data: List[LmStudioModel], object_type: str):
        self.data = data
        self.object_type = object_type

    @classmethod
    def from_json(cls, json_string: str) -> 'LmStudioModelList':
        json_data = json.loads(json_string)
        models = [LmStudioModel.from_dict(item) for item in json_data['data']]
        return cls(data=models, object_type=json_data['object'])

    def to_json(self) -> str:
        return json.dumps({
            'data': [model.to_dict() for model in self.data],
            'object': self.object_type
        })


from ai.core.ai_model_list import AiModelList
from ai.core.ai_setting import AiSettings
from ai.llm.remote.service.mistraliz import Mistraliz
from model.operation import Operation


class MistralController:

    def __init__(self, api_key: str):
        self.api_key = api_key
        self.mistraliz = Mistraliz(api_key)

    def run(
            self,
            ai_setting: AiSettings,
            prompt: str,
            image_path: str | None = None
    ) -> Operation[str]:
        if ai_setting.model_name == AiModelList.OPEN_MISTRAL:
            return self.__run_open_mistral(ai_setting.source.mistral_name, prompt)
        if ai_setting.model_name == AiModelList.PIXSTRAL:
            return self.__run_pixstral_vision(ai_setting.source.mistral_name, prompt, image_path)
        raise NotImplementedError("Model not implemented in MistralController")

    def __run_pixstral_vision(self, model_id: str, prompt: str, image_path: str | None) -> Operation[str]:
        try:
            if image_path is None:
                raise ValueError("Image path is required for Pixstral Vision")
            result = self.mistraliz.run_pixstral_vision(model_id, image_path, prompt)
            return Operation(status=True, payload=result)
        except Exception as e:
            return Operation(status=False, error=str(e))

    def __run_open_mistral(self, model_id: str, prompt: str) -> Operation[str]:
        try:
            result = self.mistraliz.run_open_mistral(model_id, prompt)
            return Operation(status=True, payload=result)
        except Exception as e:
            return Operation(status=False, error=str(e))




from ai.core.ai_method import AiMethod
from ai.core.ai_power import AiPower
from ai.core.ai_source import AiSource
from ai.core.ai_source_type import AiSourceType
from ai.core.hg_file import HgFile
from model.file_type import FileType


class AiModels:


    def __init__(self):
        pass

    # noinspection DuplicatedCode
    class Llava:

        # Local Models
        llava_15_7b_mmproj_f16 = HgFile("mmproj-model-f16.gguf", "https://huggingface.co/mys/ggml_llava-v1.5-7b/resolve/main/mmproj-model-f16.gguf", FileType.HG_MMPROJ)
        llava_15_7b_ggml_model_q4 = HgFile("ggml-model-q4_k.gguf", "https://huggingface.co/mys/ggml_llava-v1.5-7b/resolve/main/ggml-model-q4_k.gguf", FileType.HG_GGML)
        llava_15_7b_bundle = [llava_15_7b_mmproj_f16, llava_15_7b_ggml_model_q4]
        llava_15_7b_name = "llava157b"

        # LLM Studio Models
        llava_phi_3_mini_1 = "llava-phi-3-mini-int4"

        @staticmethod
        def get_llava(power: AiPower, source: AiSourceType) -> AiSource:
            if power == AiPower.LOW:
                return AiModels.Llava.get_llava_power_low(source)
            elif power == AiPower.MEDIUM:
                return AiModels.Llava.get_llava_power_medium(source)
            elif power == AiPower.HIGH:
                return AiModels.Llava.get_llava_power_high(source)
            raise Exception("No model found for the given power and method.")

        @staticmethod
        def get_llava_power_low(source: AiSourceType) -> AiSource:
            if source == AiSourceType.OLLAMA_SERVER:
                return AiSource(ollama_name="llava:7b")
            if source == AiSourceType.LOCAL_LLAMACPP:
                return AiSource(local_name=AiModels.Llava.llava_15_7b_name, hg_files=AiModels.Llava.llava_15_7b_bundle)
            if source == AiSourceType.LMSTUDIO_SERVER:
                return AiSource(lmstudio_name=AiModels.Llava.llava_phi_3_mini_1)
            raise Exception("No model found for the given power and method.")

        @staticmethod
        def get_llava_power_medium(source: AiSourceType) -> AiSource:
            if source == AiSourceType.OLLAMA_SERVER:
                return AiSource(ollama_name="llava:13b")
            if source == AiSourceType.LOCAL_LLAMACPP:
                return AiSource(local_name=AiModels.Llava.llava_15_7b_name, hg_files=AiModels.Llava.llava_15_7b_bundle)
            if source == AiSourceType.LMSTUDIO_SERVER:
                return AiSource(lmstudio_name=AiModels.Llava.llava_phi_3_mini_1)
            raise Exception("No model found for the given power and method.")

        @staticmethod
        def get_llava_power_high(source: AiSourceType) -> AiSource:
            if source == AiSourceType.OLLAMA_SERVER:
                return AiSource(ollama_name="llava:13b")
            if source == AiSourceType.LOCAL_LLAMACPP:
                return AiSource(local_name=AiModels.Llava.llava_15_7b_name, hg_files=AiModels.Llava.llava_15_7b_bundle)
            if source == AiSourceType.LMSTUDIO_SERVER:
                return AiSource(lmstudio_name=AiModels.Llava.llava_phi_3_mini_1)
            raise Exception("No model found for the given power and method.")


    class Mistral:

        @staticmethod
        def get_pixstral() -> AiSource:
            return AiSource(mistral_name="pixtral-12b-2409")

        @staticmethod
        def get_open_mistral() -> AiSource:
            return AiSource(mistral_name="open-mistral-7b")


import json
from typing import List, Dict, Any


class OllamaResponse:
    def __init__(self, model: str, created_at: str, response: str, done: bool,
                 done_reason: str, context: List[int], total_duration: int,
                 load_duration: int, prompt_eval_count: int,
                 prompt_eval_duration: int, eval_count: int, eval_duration: int):
        self.model = model
        self.created_at = created_at
        self.response = self.clean_response(response)
        self.done = done
        self.done_reason = done_reason
        self.context = context
        self.total_duration = total_duration
        self.load_duration = load_duration
        self.prompt_eval_count = prompt_eval_count
        self.prompt_eval_duration = prompt_eval_duration
        self.eval_count = eval_count
        self.eval_duration = eval_duration

    @staticmethod
    def clean_response(response: str) -> str:
        # Remove the backticks and the "json" prefix from the response string
        cleaned_response = response.strip("`")
        if cleaned_response.lower().startswith("json"):
            cleaned_response = cleaned_response[4:].strip()
        # Remove the remaining backticks and newline characters
        cleaned_response = cleaned_response.replace("```", "").strip()
        return cleaned_response.replace("json", "")

    @classmethod
    def from_json(cls, data: Dict[str, Any]) -> 'OllamaResponse':
        return cls(
            # TODO se da problemi mettere none a tutti i get
            model=data.get("model", ""),
            created_at=data.get("created_at", ""),
            response=data.get("response", ""),
            done=data.get("done", False),
            done_reason=data.get("done_reason", ""),
            context=data.get("context", []),
            total_duration=data.get("total_duration", 0),
            load_duration=data.get("load_duration", 0),
            prompt_eval_count=data.get("prompt_eval_count", 0),
            prompt_eval_duration=data.get("prompt_eval_duration", 0),
            eval_count=data.get("eval_count", 0),
            eval_duration=data.get("eval_duration", 0)
        )

    def __str__(self) -> str:
        return (
            f"Model: {self.model}\n"
            f"Created At: {self.created_at}\n"
            f"Response: {self.response}\n"
            f"Done: {self.done}\n"
            f"Done Reason: {self.done_reason}\n"
            f"Context: {self.context}\n"
            f"Total Duration: {self.total_duration}\n"
            f"Load Duration: {self.load_duration}\n"
            f"Prompt Eval Count: {self.prompt_eval_count}\n"
            f"Prompt Eval Duration: {self.prompt_eval_duration}\n"
            f"Eval Count: {self.eval_count}\n"
            f"Eval Duration: {self.eval_duration}\n"
        )


import unittest

from ai.util.ai_runner import AiRunner
from ai.core.ai_inputs import AiInputs
from ai.core.ai_model_list import AiModelList
from ai.core.ai_power import AiPower
from ai.core.ai_prompts import AiPrompt
from ai.core.ai_setting import AiSettings
from ai.core.ai_source_type import AiSourceType
import sys
import os
from dotenv import load_dotenv
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


class TestLmStudio(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        print("Setting up test...")


    def test1(self):
        setting = AiSettings(
            model=AiModelList.OPEN_MISTRAL,
            source_type=AiSourceType.API_MISTRAL,
            power=AiPower.LOW,
            api_key=os.getenv('MISTRAL_API_KEY'),
        )
        inputs = AiInputs(prompt="Why is the sky blue? answer in 20 words or less")
        result = AiRunner(setting, inputs).run()
        print(result.payload)


    def test2(self):
        setting = AiSettings(
            model=AiModelList.PIXSTRAL,
            source_type=AiSourceType.API_MISTRAL,
            power=AiPower.MEDIUM,
            api_key=os.getenv('MISTRAL_API_KEY'),
        )
        inputs = AiInputs(prompt=AiPrompt.VISION_DETAILED.value, file_path=os.getenv('LOCAL_IMAGE_FOR_TEST'))
        result = AiRunner(setting, inputs).run()
        print("Local image for test: ", os.getenv('LOCAL_IMAGE_FOR_TEST'))
        print(f"Result status: {result.status}")
        print("Result error: ", result.error)
        print("Result payload: ", result.payload)




if __name__ == "__main__":
    unittest.main()


import json
from typing import Optional, List
from datetime import datetime


class Details:
    def __init__(self, parent_model: str, format: str, family: str, families: Optional[List[str]], parameter_size: str, quantization_level: str) -> None:
        self.parent_model = parent_model
        self.format = format
        self.family = family
        self.families = families
        self.parameter_size = parameter_size
        self.quantization_level = quantization_level

    def to_json(self) -> dict:
        return {
            "parent_model": self.parent_model,
            "format": self.format,
            "family": self.family,
            "families": self.families,
            "parameter_size": self.parameter_size,
            "quantization_level": self.quantization_level
        }

    @classmethod
    def from_json(cls, data: dict):
        return cls(
            parent_model=data.get("parent_model", ""),
            format=data.get("format", ""),
            family=data.get("family", ""),
            families=data.get("families", []),
            parameter_size=data.get("parameter_size", ""),
            quantization_level=data.get("quantization_level", "")
        )


class OllamaModel:
    def __init__(self, name: str, model: str, modified_at: datetime, size: int, digest: str, details: Details) -> None:
        self.name = name
        self.model = model
        self.modified_at = modified_at
        self.size = size
        self.digest = digest
        self.details = details

    def to_json(self) -> dict:
        return {
            "name": self.name,
            "model": self.model,
            "modified_at": self.modified_at.isoformat(),
            "size": self.size,
            "digest": self.digest,
            "details": self.details.to_json()
        }

    @classmethod
    def from_json(cls, data: dict):
        return cls(
            name=data.get("name", ""),
            model=data.get("model", ""),
            modified_at=datetime.fromisoformat(data.get("modified_at")),
            size=data.get("size", 0),
            digest=data.get("digest", ""),
            details=Details.from_json(data.get("details", {}))
        )


class OllamaModels:
    def __init__(self, models: List[OllamaModel]) -> None:
        self.models = models

    def to_json(self) -> dict:
        return {
            "models": [model.to_json() for model in self.models]
        }

    @classmethod
    def from_json(cls, data: dict):
        return cls(
            models=[OllamaModel.from_json(model_data) for model_data in data.get("models", [])]
        )





import json
from typing import Callable

from ai.llm.remote.data.ollamapi import OllamaApiLegacy, Ollamapi

from ai.llm.remote.dto.ollama_models import OllamaModels
from ai.llm.remote.dto.ollama_response import OllamaResponse
from model.operation import Operation


class OllamalizLegacy:

    def __init__(self):
        pass

    @staticmethod
    def check_ollama(url: str):
        response = OllamaApiLegacy.check_ollama_status(url)
        if response.is_successful():
            return
        else:
            error = response.get_error()
            raise Exception(error)


class Ollamaliz:

    def __init__(self, url: str):
        self.obj = Ollamapi(url)
        pass

    def get_models_list(self) -> OllamaModels:
        response = self.obj.get_models_list()
        data = json.loads(response)
        return OllamaModels.from_json(data)

    def has_model(self, name: str):
        models = self.get_models_list().models
        for model in models:
            if model.name == name:
                return True
        return False

    def download_model(self, name: str, en_stream: bool, callback: Callable[[str], None] | None = None):
        return self.obj.download_model(name, en_stream, callback)

    def llava_query(
            self,
            prompt: str,
            image_base_64: str,
            model_name: str,
    ) -> Operation[OllamaResponse]:
        response = OllamaApiLegacy.send_llava_query(self.obj.url, prompt, image_base_64, model_name)
        if response.is_successful():
            resp_text = response.text
            resp_text_json = json.loads(resp_text)
            resp_obj = OllamaResponse.from_json(resp_text_json)
            return Operation(payload=resp_obj, status=True)
        else:
            error = response.get_error()
            return Operation(status=False, error=error)








from network.netres import NetResponse
from network.netutils import exec_get

LMSTUDIO_PORT = "1234"
LMSTUDIO_HTTP_LOCALHOST_URL = "http://localhost:" + LMSTUDIO_PORT


class LmmStudioApi:

    def __init__(self, url: str):
        self.url = url

    def get_ram_loaded_models(self) -> NetResponse:
        api_url = self.url + "/v1/models"
        return exec_get(api_url)



import base64

from media.api.data.eagleapi import EAGLE_LOCALHOST_URL, EagleApi
from media.api.dto.eagle_dto import EagleDto
from old_code.liz_image import LizImage
from model.operation import Operation


class Eagleliz:
    def __init__(self, url: str = EAGLE_LOCALHOST_URL):
        self.obj = EagleApi(url)

    def get_app_info(self) -> Operation[EagleDto]:
        response = self.obj.get_app_info()
        if response.is_successful():
            resp_json: str = response.json
            eagle_obj = EagleDto.from_dict(resp_json)
            return Operation(status=True, payload=eagle_obj)
        else:
            error = response.get_error()
            return Operation(status=False, error=error)

    def add_image_from_path(self, image: LizImage) -> Operation[None]:
        # Converting image to base64
        with open(image.path, "rb") as image_file:
            encoded_string = base64.b64encode(image_file.read()).decode('utf-8')
        # calling api
        resp_obj = self.obj.add_image_from_path(
            path=image.path,
            name=image.ai_file_name,
            tags=image.ai_tags,
            annotation=image.get_desc_plus_text(),
            modification_time=image.creation_time_timestamp,
        )
        # checking response
        if resp_obj.is_successful():
            resp_json: str = resp_obj.json
            eagle_obj = EagleDto.from_dict2(resp_json)
            if eagle_obj.status == "success":
                return Operation(status=True)
            else:
                return Operation(status=False, error="Error while adding image to eagle.")
        else:
            error = resp_obj.get_error()
            return Operation(status=False, error=error)


from enum import Enum


class AiPower(Enum):
    LOW = "Low"
    MEDIUM = "Medium"
    HIGH = "High"

    @staticmethod
    def get_llava_from_power(ai_power: str) -> str:
        if ai_power == AiPower.HIGH.value:
            return "llava:13b"
        elif ai_power == AiPower.MEDIUM.value:
            return "llava:13b"
        else:
            return "llava:7b"



import json

from ai.llm.remote.data.lmstudioapi import LmmStudioApi
from ai.llm.remote.dto.lmstudio_models import LmStudioModelList


class LmStudioLiz:

    def __init__(self, url: str):
        self.obj = LmmStudioApi(url)
        pass

    def get_loaded_models(self) -> LmStudioModelList:
        call = self.obj.get_ram_loaded_models()
        if call.is_error():
            raise Exception(call.get_error())
        return LmStudioModelList.from_json(call.response.text)

    def has_model_loaded(self, model_id: str) -> bool:
        models = self.get_loaded_models()
        for model in models.data:
            if model.id == model_id:
                return True
        return False

    def send_vision_prompt(self, image_base_64: str, prompt: str) -> str:
        raise NotImplementedError("Not implemented yet.")
